package rocks.zipcode.atm;

import javafx.scene.control.Alert;
import javafx.scene.control.PasswordField;
import javafx.scene.control.TextInputDialog;

import java.util.Optional;

/**
 * @author ZipCodeWilmington
 */
public class ActionResult<T> {

    private T data;
    private String errorMessage;

    private ActionResult(T data) {
        this.data = data;
    }

    private ActionResult(String errorMessage) {
        this.errorMessage = errorMessage;
    }

    public T getData() {
        return data;
    }

    public String getErrorMessage() {
        return errorMessage;
    }

    public boolean isSuccess() {
        return data != null;
    }

    public static <E> ActionResult<E> success(E data) {
        return new ActionResult<E>(data);
    }

//    public static <E> ActionResult<E> fail(String errorMessage) {
//        return new ActionResult<E>(errorMessage);
//    }

    public static void fail(String errorMessage) {
        Alert alert = new Alert(Alert.AlertType.ERROR);
        alert.setTitle("Error Dialog");
        alert.setHeaderText("An error has occurred!");
        alert.setContentText(errorMessage);

        alert.showAndWait();
    }

    public static Optional<String> getInfo(String title, String text) {
        TextInputDialog textDialog = new TextInputDialog();
        textDialog.setTitle(title);
        textDialog.setHeaderText(null);

        textDialog.setContentText(text);
        return textDialog.showAndWait();
    }

    public static String getPassword(String prompt) {
        PasswordField passwordField = new PasswordField();
        passwordField.setPromptText(prompt);

        return passwordField.getText();
    }

}
