package rocks.zipcode.atm.bank;

import javafx.scene.control.TextInputDialog;
import rocks.zipcode.atm.ActionResult;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;
import java.util.Scanner;

/**
 * @author ZipCodeWilmington
 */
public class Bank {

    private Map<Integer, Account> accounts = new HashMap<>();
    private String name;
    private String email;

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    public Bank() {
//
//        this.name = "No name available";
//        this.email = "No e-mail available";

        accounts.put(1000, new BasicAccount(new AccountData(1000, "Example 1", "example1@gmail.com", 500)));

        accounts.put(2000, new PremiumAccount(new AccountData(
                2000, "Example 2", "example2@gmail.com", 200)));
    }

    public ActionResult<AccountData> getAccountById(int id) {
        Account account = accounts.get(id);

        ActionResult result;

        if (account != null) {
            result = ActionResult.success(account.getAccountData());
        } else {
            result = null;
            ActionResult.fail("Account ID " + id + " does not exist. Please create new account.");
        }

        return result;
    }

    public ActionResult<AccountData> createAccountById(int id) {
        Account account = accounts.get(id);

        ActionResult result;

        if (account == null) {

            Optional<String> newName = ActionResult.getInfo("Set Name", "Enter name: ");
            Optional<String> newEmail = ActionResult.getInfo("Set E-mail", "Enter e-mail: ");
            Optional<String> initialDeposit = ActionResult.getInfo("Initial Deposit", "Enter initial deposit: ");

            // Fetch user input
            AccountData accountData = new AccountData(id, newName.get(), newEmail.get(), Integer.parseInt(initialDeposit.get()));
            Account newAccount = new Account(accountData);

            accounts.put(id, newAccount);

            result = ActionResult.success(newAccount.getAccountData());
        } else {
            result = null;
            ActionResult.fail("Account ID " + id + " already exists. Please try again.");
        }

        return result;
    }

    public ActionResult<AccountData> editAccountName1(int id) {
        Account account = accounts.get(id);
        AccountData accountInfo = account.getAccountData();

        Optional<String> newName = ActionResult.getInfo("Edit name", "Enter new name: ");

        AccountData accountData = new AccountData(id, newName.get(), accountInfo.getEmail(), accountInfo.getBalance());
        Account newAccount = new Account(accountData);

        accounts.put(id, newAccount);

        return ActionResult.success(newAccount.getAccountData());
    }

//    public ActionResult<AccountData> editAccountName(String name) {
//
//        AccountData accountData = new AccountData(55, name, "akjsdkas@kasd.com", 500);
//
//        Account account = new Account(accountData);
//
//        accounts.put(55, account);
//
//        return ActionResult.success(account.getAccountData());
//    }

    public ActionResult<AccountData> editAccountEmail(int id) {
        Account account = accounts.get(id);
        AccountData accountInfo = account.getAccountData();

        Optional<String> newEmail = ActionResult.getInfo("Edit E-mail", "Enter new e-mail: ");

        AccountData accountData = new AccountData(id, accountInfo.getName(), newEmail.get(), accountInfo.getBalance());
        Account newAccount = new Account(accountData);

        accounts.put(id, newAccount);

        return ActionResult.success(newAccount.getAccountData());
    }

    public ActionResult<AccountData> deposit(int id) {
        Account account = accounts.get(id);
        AccountData accountInfo = account.getAccountData();

        Optional<String> depositAmount = ActionResult.getInfo("Deposit into Account", "Enter amount to deposit: ");

        AccountData accountData = new AccountData(id, accountInfo.getName(), accountInfo.getEmail(), accountInfo.getBalance() + Integer.parseInt(depositAmount.get()));
        Account newAccount = new Account(accountData);

        accounts.put(id, newAccount);

        return ActionResult.success(newAccount.getAccountData());
    }

    public ActionResult<AccountData> withdraw(int id) {
        Account account = accounts.get(id);
        AccountData accountInfo = account.getAccountData();

        Optional<String> withdrawAmount = ActionResult.getInfo("Withdraw from Account", "Enter amount to withdraw: ");

        int amount = Integer.parseInt(withdrawAmount.get());

        boolean ok = account.withdraw(amount);

        ActionResult result;

        if (ok) {
            AccountData accountData = new AccountData(id, accountInfo.getName(), accountInfo.getEmail(), accountInfo.getBalance() - Integer.parseInt(withdrawAmount.get()));
            Account newAccount = new Account(accountData);

            accounts.put(id, newAccount);

            result = ActionResult.success(newAccount.getAccountData());
        } else {
            result = null;
            ActionResult.fail("Withdraw failed: " + amount + ". Account has: " + account.getBalance());
        }

        return result;
    }
}
